/*
 * Copyright (C) 2004, no1sm [at] narod [dot] ru
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <fcntl.h>
#include <sys/poll.h>
#include <getopt.h>
#include <sys/ioctl.h>
#include <errno.h>

void usage(const char *progname)
{
    printf("Usage: %s [options] -c command\n"
	   "      -b, beeper (/dev/beep)\n"
	   "      -d, be daemon\n"
	   "      -c, command to execute on beep\n"
	   "      -h, this meaningful display\n", progname);
}

int main(int argc, char *argv[])
{
    struct pollfd pollfd;
    int beep, c;
    int be_daemon = 0;
    char * dev = "/dev/beep";
    char * cmd = NULL;
    
    while ((c = getopt(argc, argv, "db:c:h")) > 0) {
	switch (c) {
	case 'c':
	    cmd = strdup(optarg);
	    break;
	case 'b':
	    dev = strdup(optarg);
	    break;
	case 'd':
	    be_daemon = 1;
	    break;
	case 'h':
	    usage(argv[0]);
	    exit(0);
	default:
	    usage(argv[0]);
	    exit(1);
	}
    }
    
    if(!cmd) {
	usage(argv[0]);
	exit(1);
    }
    
    if((beep = open(dev, O_RDWR)) < 0) {
	perror("open");
	exit(1);
    }

    if(be_daemon) {
	int null, n;
	pid_t pid = fork();
	
	if(pid < 0) {
	    perror("fork");
	    exit(1);
	} else if(pid) { /* parent */
	    exit(0);
	}

	signal(SIGHUP, SIG_IGN);
	signal(SIGCHLD, SIG_IGN);
	signal(SIGTTOU, SIG_IGN);
	signal(SIGTTIN, SIG_IGN);

	null = open("/dev/null", O_RDWR);
	close(0);
	close(1);
	close(2);
	dup2(null, 0);
	dup2(0, 1);
	dup2(0, 2);
	close(null);
	/* detach from console */
	n = open("/dev/tty", O_RDWR);
	if (n >= 0) {
	    ioctl(n, TIOCNOTTY, (char *) 0);
	    close(n);
	}
    }

    pollfd.fd 	  = beep;
    pollfd.events = POLLIN;

    while(1) {
	if(poll(&pollfd, 1, -1) < 0) {
	    if(errno == EAGAIN || errno == EINTR)
		continue;
	    exit(1);
	}
	if(pollfd.events |= POLLIN)
	    system(cmd);
    }
    
    return 0;
}
